#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <ctype.h>

#include "parser.h"
#include "extstring.h"

#ifdef DEBUG
	#include "printtree.h"
#endif

tSyntaxElement* parse(FILE* file) {

	tSyntaxElement* syntaxtree = se_create();
	tSyntaxElement* syntaxtreestart = syntaxtree;
	syntaxtree->type = NEWTREE;

	char symbol;
	bool intree = false;
	bool incomment = false;
	bool instring = false;
	bool intoken = false;

	tExtstring* exttoken = NULL;
	tExtstring* extstring = NULL;

	while (1) {
		symbol = fgetc(file);

		#ifdef DEBUG

			if (!isspace(symbol))
				printf("In parsing. Current symbol: %c\n", symbol);
			else
				printf("In parsing. Current symbol: %d\n", symbol);

			printtree(syntaxtreestart, 1);

		#endif

		if (feof(file))
			break;

		if (incomment) {
			if (symbol == '\n')
				incomment = false;
			continue;
		}

		if (instring) {
			if (symbol == '"') {

				if (extstring == NULL) {
					extstring = malloc(sizeof(tExtstring));
					extstring->next = NULL;
				}

				instring = false;

				syntaxtree->type = STRING;
				char* string = es_tostring(extstring);
				syntaxtree->content.string = string;

				es_free(extstring);
				extstring = NULL;

			} else {

				if (extstring == NULL) {

					extstring = malloc(sizeof(tExtstring));
					extstring->symbol = symbol;
					extstring->next = NULL;

					continue;
				}

				es_addsymbol(extstring, symbol);
			}
			continue;
		}
		if (intoken) {
			if (isspace(symbol) || symbol == '(' || symbol == ')') {

				intoken = false;

				syntaxtree->type = TOKEN;
				char* string = es_tostring(exttoken);
				syntaxtree->content.string = string;

				es_free(exttoken);
				exttoken = NULL;

			} else {

				es_addsymbol(exttoken, symbol);

				continue;
			}
		}

		if (isspace(symbol)) 
			continue;

		if (intree) {
			switch (symbol) {

				case '(':

					syntaxtree = se_next(syntaxtree);
					syntaxtree->type = TREE;

					syntaxtree = se_bottom(syntaxtree);
					syntaxtree->type = NEWTREE;

					break;

				case ')':

					syntaxtree = syntaxtree->top;

					if (syntaxtree->top == NULL)
						intree = false;
					break;

				case ';':

					incomment = true;
					continue;

				case '"':

					syntaxtree = se_next(syntaxtree);
					instring = true;
					continue;

				default:

					syntaxtree = se_next(syntaxtree);

					exttoken = malloc(sizeof(tExtstring));
					exttoken->symbol = symbol;
					exttoken->next = NULL;

					intoken = true;
					break;
			}
			continue;

		} else {
			switch (symbol) {

				case ')':

					fprintf(stderr, "Unbalanced brackets.\n");
					return NULL;

				case ';':

					incomment = true;
					continue;

				case '(':

					intree = true;

					syntaxtree = se_next(syntaxtree);
					syntaxtree->type = TREE;

					syntaxtree = se_bottom(syntaxtree);
					syntaxtree->type = NEWTREE;

					continue;

				default:

					fprintf(stderr, "Expression outside of brackets.\n");
					return NULL;
			}
		}
	}

	if (intree) {

		fprintf(stderr, "Unexpected EOF.\n");
		return NULL;

	}

	return syntaxtreestart;
}
